@@ -67,7 +67,7 @@ class EasyXmlTree:
 
 		return result
 
-	def tostring(self) -> str:
+	def to_string(self) -> str:
 		"""
 		Serialize the tree to a string.
 		"""
@@ -90,12 +90,12 @@ class EasyXhtmlTree(EasyXmlTree):
 
 		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", ""))
 
-	def tostring(self) -> str:
+	def to_string(self) -> str:
 		"""
 		Serialize the tree to a string.
 		"""
 
-		xml = EasyXmlTree.tostring(self)
+		xml = EasyXmlTree.to_string(self)
 
 		xml = xml.replace("<html", "<html xmlns=\"http://www.w3.org/1999/xhtml\"")
 
@@ -112,12 +112,12 @@ class EasySvgTree(EasyXmlTree):
 
 		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.w3.org/2000/svg\"", ""))
 
-	def tostring(self) -> str:
+	def to_string(self) -> str:
 		"""
 		Serialize the tree to a string.
 		"""
 
-		xml = EasyXmlTree.tostring(self)
+		xml = EasyXmlTree.to_string(self)
 
 		xml = xml.replace("<svg", "<svg xmlns=\"http://www.w3.org/2000/svg\"")
 
@@ -134,12 +134,12 @@ class EasyOpfTree(EasyXmlTree):
 
 		EasyXmlTree.__init__(self, xml_string.replace(" xmlns=\"http://www.idpf.org/2007/opf\"", ""))
 
-	def tostring(self) -> str:
+	def to_string(self) -> str:
 		"""
 		Serialize the tree to a string.
 		"""
 
-		xml = EasyXmlTree.tostring(self)
+		xml = EasyXmlTree.to_string(self)
 
 		xml = xml.replace("<package", "<package xmlns=\"http://www.idpf.org/2007/opf\"")
 
@@ -153,7 +153,7 @@ class EasyXmlElement:
 	def __init__(self, lxml_element):
 		self.lxml_element = lxml_element
 
-	def totagstring(self) -> str:
+	def to_tag_string(self) -> str:
 		"""
 		Return a string representing the opening tag of the element.
 
@@ -172,7 +172,7 @@ class EasyXmlElement:
 
 		return f"<{self.lxml_element.tag}{attrs}>"
 
-	def tostring(self) -> str:
+	def to_string(self) -> str:
 		"""
 		Return a string representing this element.
 
@@ -183,7 +183,7 @@ class EasyXmlElement:
 
 		return regex.sub(r" xmlns(:[\p{Letter}]+?)?=\"[^\"]+?\"", "", etree.tostring(self.lxml_element, encoding=str, with_tail=False))
 
-	def attribute(self, attribute: str) -> str:
+	def get_attr(self, attribute: str) -> str:
 		"""
 		Return the value of an attribute on this element.
 		"""
@@ -192,6 +192,15 @@ class EasyXmlElement:
 
 		return self.lxml_element.get(attribute)
 
+	def set_attr(self, attribute: str, value: str) -> str:
+		"""
+		Set the value of an attribute on this element.
+		"""
+
+		attribute = attribute.replace("epub:", "{http://www.idpf.org/2007/ops}")
+
+		return self.lxml_element.set(attribute, value)
+
 	def xpath(self, selector: str, return_string: bool = False):
 		"""
 		Shortcut to select elements based on xpath selector.
@@ -224,7 +233,7 @@ class EasyXmlElement:
 		`<p>Hello there, <abbr>Mr.</abbr> Smith!</p>` -> `Hello there, <abbr>Mr.</abbr> Smith!`
 		"""
 
-		xml = self.tostring()
+		xml = self.to_string()
 		xml = regex.sub(r"^<[^>]+?>", "", xml)
 		xml = regex.sub(r"<[^>]+?>$", "", xml)
 		return xml
@@ -305,6 +314,33 @@ class EasyXmlElement:
 		self.lxml_element.addnext(node.lxml_element)
 		self.remove()
 
+	def append(self, node) -> None:
+		"""
+		Place node as the last child of this node.
+		"""
+
+		if isinstance(node, EasyXmlElement):
+			self.lxml_element.append(node.lxml_element)
+
+		else:
+			self.lxml_element.append(node)
+
+	@property
+	def tag(self) -> str:
+		"""
+		Return a string representing this node's tag name, like `body` or `section`
+		"""
+
+		return self.lxml_element.tag
+
+	@property
+	def parent(self): # This returns an EasyXmlElement but we can't type hint this until Python 3.10
+		"""
+		Return an EasyXmlElement representing this node's parent node
+		"""
+
+		return EasyXmlElement(self.lxml_element.getparent())
+
 	@property
 	def text(self) -> str:
 		"""
