@@ -129,7 +129,7 @@ def semanticate(xhtml: str) -> str:
 	xhtml = regex.sub(r"""([^\p{Letter}>\"])([vxVX])(\b|st\b|nd\b|rd\b|th\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
 
 	# We can assume a lowercase i is always a Roman numeral
-	xhtml = regex.sub(r"""([^\p{Letter}>\"])(i)(\b|st\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
+	xhtml = regex.sub(r"""([^\p{Letter}<>/\"])(i)(\b|st\b)""", r"""\1<span epub:type="z3998:roman">\2</span>\3""", xhtml)
 
 	# Fix X-ray
 	xhtml = regex.sub(r"""<span epub:type="z3998:roman">([Xx])</span>-ray""", r"""\1-ray""", xhtml)
@@ -138,7 +138,7 @@ def semanticate(xhtml: str) -> str:
 	xhtml = regex.sub(r"([0-9]+)\s*([cmk][mgl])\b", fr"\1{se.NO_BREAK_SPACE}<abbr>\2</abbr>", xhtml)
 
 	# Add abbrevations around Imperial measurements
-	xhtml = regex.sub(r"([0-9]+)\s*(ft|in|yd|mi|pt|qt|gal|oz|lbs)\.?\b", fr"\1{se.NO_BREAK_SPACE}<abbr>\2.</abbr>", xhtml)
+	xhtml = regex.sub(r"(?<![\$£0-9])([0-9]+)\s*(ft|in|yd|mi|pt|qt|gal|oz|lbs)\.?\b", fr"\1{se.NO_BREAK_SPACE}<abbr>\2.</abbr>", xhtml)
 
 	# Tweak some other Imperial measurements
 	xhtml = regex.sub(r"([0-9]+)\s*m\.?p\.?h\.?", fr"\1{se.NO_BREAK_SPACE}<abbr>mph</abbr>", xhtml, flags=regex.IGNORECASE)
@@ -1058,7 +1058,7 @@ def titlecase(text: str) -> str:
 	text = regex.sub(r"(?:vs\.) The\b", "vs. the", text)
 
 	# Lowercase "de", "von", "van", "le", as in "Charles de Gaulle", "Werner von Braun", etc., and if not the first word and not preceded by an &ldquo;
-	text = regex.sub(r"(?<!^|“)\b(De|Von|Van|Le)\b", lambda result: result.group(1).lower(), text)
+	text = regex.sub(r"(?<!^|“)\b(Von|Van|Le|Des|De La|De)\b", lambda result: result.group(1).lower(), text)
 
 	# Uppercase word following "Or,", since it is probably a subtitle
 	text = regex.sub(r"\bOr, ([\p{Lowercase_Letter}])", lambda result: "Or, " + result.group(1).upper(), text)
@@ -1096,7 +1096,7 @@ def titlecase(text: str) -> str:
 	text = regex.sub(r"(?<!^)\bÀ\b", "à", text)
 
 	# Uppercase initialisms
-	text = regex.sub(r"(\s)(([\p{Letter}]\.)+)", lambda result: result.group(1) + result.group(2).upper(), text)
+	text = regex.sub(r"\b(([\p{Letter}]\.)+)", lambda result: result.group(1).upper(), text)
 
 	# Fix html entities
 	text = text.replace("&Amp;", "&amp;")
@@ -1223,10 +1223,13 @@ def generate_title(xhtml) -> str:
 	A string representing the title for the document
 	"""
 
-	if isinstance(xhtml, str):
-		dom = EasyXhtmlTree(xhtml)
-	else:
-		dom = deepcopy(xhtml)
+	try:
+		if isinstance(xhtml, str):
+			dom = EasyXhtmlTree(xhtml)
+		else:
+			dom = deepcopy(xhtml)
+	except Exception as ex:
+		raise se.InvalidXhtmlException(f"Couldn’t parse XHTML file. Exception: {ex}")
 
 	title = ""
 
@@ -1249,9 +1252,9 @@ def generate_title(xhtml) -> str:
 			raise se.InvalidSeEbookException("No [xhtml]<section>[/] or [xhtml]<article>[/] element for [xhtml]<hgroup>[/]")
 
 		# If the closest parent <section> or <article> is a part, division, or volume, then keep all <hgroup> children
-		if closest_parent_section.attribute("epub:type") and ("part" not in closest_parent_section.attribute("epub:type") and "division" not in closest_parent_section.attribute("epub:type") and "volume" not in closest_parent_section.attribute("epub:type")):
+		if closest_parent_section.get_attr("epub:type") and ("part" not in closest_parent_section.get_attr("epub:type") and "division" not in closest_parent_section.get_attr("epub:type") and "volume" not in closest_parent_section.get_attr("epub:type")):
 			# Else, if the closest parent <section> or <article> is a halftitlepage, then discard <hgroup> subtitles
-			if closest_parent_section.attribute("epub:type") and "halftitlepage" in closest_parent_section.attribute("epub:type"):
+			if closest_parent_section.get_attr("epub:type") and "halftitlepage" in closest_parent_section.get_attr("epub:type"):
 				for node in hgroup_element.xpath("./*[contains(@epub:type, 'subtitle')]"):
 					node.remove()
 
@@ -1296,9 +1299,9 @@ def generate_title(xhtml) -> str:
 				top_level_wrapper = top_level_wrappers[0]
 
 				# Only guess the title if there is a single value for epub:type
-				if top_level_wrapper.attribute("epub:type"):
+				if top_level_wrapper.get_attr("epub:type"):
 					# Get the first non-namespaced value as the title
-					for value in top_level_wrapper.attribute("epub:type").split(" "):
+					for value in top_level_wrapper.get_attr("epub:type").split(" "):
 						if ":" not in value:
 							title = titlecase(value.replace("-", " "))
 							break
