@@ -8,12 +8,10 @@ import base64
 import concurrent.futures
 import datetime
 import fnmatch
-import html
 import os
 from pathlib import Path
 from typing import List, Optional, Tuple, Union
 
-from bs4 import BeautifulSoup, Tag
 import git
 from natsort import natsorted
 import regex
@@ -68,6 +66,7 @@ class Endnote:
 	"""
 
 	def __init__(self):
+		self.node = None
 		self.number = 0
 		self.anchor = ""
 		self.contents = []  # The strings and tags inside an <li> element
@@ -91,7 +90,7 @@ class SeEpub:
 	_generated_github_repo_url = None
 	_repo = None # git.Repo object
 	_last_commit = None # GitCommit object
-	__endnotes_soup = None # bs4 soup object of the endnotes.xhtml file
+	__endnotes_dom = None # EasyXhtmlTree object of the endnotes.xhtml file
 	_endnotes: Optional[List[Endnote]] = None # List of Endnote objects
 
 	def __init__(self, epub_root_directory: Union[str, Path]):
@@ -183,7 +182,7 @@ class SeEpub:
 			translators_have_display_seq = False
 			illustrators_have_display_seq = False
 			for role in self.metadata_dom.xpath("/package/metadata/meta[@property=\"role\"]"):
-				contributor_id = role.attribute("refines").lstrip("#")
+				contributor_id = role.get_attr("refines").lstrip("#")
 				contributor_element = self.metadata_dom.xpath("/package/metadata/dc:contributor[@id=\"" + contributor_id + "\"]")
 				if contributor_element:
 					contributor = {"name": contributor_element[0].text, "include": True, "display_seq": None}
@@ -245,27 +244,27 @@ class SeEpub:
 		return self._generated_github_repo_url
 
 	@property
-	def _endnotes_soup(self) -> BeautifulSoup:
+	def _endnotes_dom(self) -> se.easy_xml.EasyXhtmlTree:
 		"""
 		Accessor
 
-		Return a BeautifulSoup object representing the endnotes.xhtml file for this ebook.
+		Return an EasyXmlTree object representing the endnotes.xhtml file for this ebook.
 
 		INPUTS
 		None
 
 		OUTPUTS
-		A BeautifulSoup object representing the endnotes.xhtml file for this ebook.
+		A EasyXmlTree object representing the endnotes.xhtml file for this ebook.
 		"""
 
-		if not self.__endnotes_soup:
+		if not self.__endnotes_dom:
 			try:
 				with open(self.path / "src" / "epub" / "text" / "endnotes.xhtml") as file:
-					self.__endnotes_soup = BeautifulSoup(file.read(), "html.parser")
+					self.__endnotes_dom = se.formatting.EasyXhtmlTree(file.read())
 			except Exception as ex:
 				raise se.InvalidFileException(f"Could't open file: [path][link=file://{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}]{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}[/][/].") from ex
 
-		return self.__endnotes_soup
+		return self.__endnotes_dom
 
 	@property
 	def endnotes(self) -> list:
@@ -284,23 +283,15 @@ class SeEpub:
 		if not self._endnotes:
 			self._endnotes = []
 
-			ol_tag: BeautifulSoup = self._endnotes_soup.find("ol")
-			items = ol_tag.find_all("li")
-
-			for item in items:
+			for node in self._endnotes_dom.xpath("/html/body/section[contains(@epub:type, 'endnotes')]/ol/li[contains(@epub:type, 'endnote')]"):
 				note = Endnote()
-				note.contents = []
-				for content in item.contents:
-					note.contents.append(content)
-					if isinstance(content, Tag):
-						links = content.find_all("a")
-						for link in links:
-							epub_type = link.get("epub:type") or ""
-							if epub_type == "backlink":
-								href = link.get("href") or ""
-								if href:
-									note.back_link = href
-				note.anchor = item.get("id") or ""
+				note.node = node
+				note.number = int(node.get_attr("id").replace("note-", ""))
+				note.contents = node.xpath("./*")
+				note.anchor = node.get_attr("id") or ""
+
+				for back_link in node.xpath("//a[contains(@epub:type, 'backlink')]/@href"):
+					note.back_link = back_link
 
 				self._endnotes.append(note)
 
@@ -320,61 +311,57 @@ class SeEpub:
 
 		return self._metadata_dom
 
-	@staticmethod
-	def _new_bs4_tag(section: Tag, output_soup: BeautifulSoup) -> Tag:
-		"""
-		Helper function used in self._recompose_xhtml()
-		Create a new BS4 tag given the current section.
-
-		INPUTS
-		section: A BS4 tag
-		output_soup: A BS4 object representing the entire soup
-
-		OUTPUTS
-		A new BS4 tag.
-		"""
-
-		tag = output_soup.new_tag(section.name)
-		for name, value in section.attrs.items():
-			tag.attrs[name] = value
-
-		return tag
-
-	def _recompose_xhtml(self, section: Tag, output_soup: BeautifulSoup) -> None:
+	def _recompose_xhtml(self, section: se.easy_xml.EasyXmlElement, output_dom: se.easy_xml.EasyXmlTree) -> None:
 		"""
 		Helper function used in self.recompose()
 		Recursive function for recomposing a series of XHTML files into a single XHTML file.
 
 		INPUTS
-		section: A BS4 tag to inspect
-		output_soup: A BS4 object representing the entire soup
+		section: An EasyXmlElement to inspect
+		output_dom: A EasyXmlTree representing the entire output dom
 
 		OUTPUTS
 		None
 		"""
 
 		# Quick sanity check before we begin
-		if "id" not in section.attrs or (section.parent.name.lower() != "body" and "id" not in section.parent.attrs):
+		if not section.get_attr("id") or (section.parent.lxml_element.tag != "body" and not section.parent.get_attr("id")):
 			raise se.InvalidXhtmlException("Section without [attr]id[/] attribute.")
 
 		# Try to find our parent tag in the output, by ID.
 		# If it's not in the output, then append it to the tag's closest parent by ID (or <body>), then iterate over its children and do the same.
-		existing_section = output_soup.select("#" + section["id"])
+		existing_section = output_dom.xpath(f"//*[@id='{section.get_attr('id')}']")
 		if not existing_section:
-			if section.parent.name.lower() == "body":
-				output_soup.body.append(self._new_bs4_tag(section, output_soup))
+			if section.parent.tag.lower() == "body":
+				output_dom.xpath("/html/body")[0].append(section)
 			else:
-				output_soup.select("#" + section.parent["id"])[0].append(self._new_bs4_tag(section, output_soup))
+				output_dom.xpath(f"//*[@id='{section.parent.get_attr('id')}']")[0].append(section)
 
-			existing_section = output_soup.select("#" + section["id"])
+			existing_section = output_dom.xpath(f"//*[@id='{section.get_attr('id')}']")
 
-		for child in section.children:
-			if not isinstance(child, str):
-				tag_name = child.name.lower()
-				if tag_name in ("section", "article"):
-					self._recompose_xhtml(child, output_soup)
-				else:
-					existing_section[0].append(child)
+		# Convert all <img> references to inline base64
+		# We even convert SVGs instead of inlining them, because CSS won't allow us to style inlined SVGs
+		# (for example if we want to apply max-width or filter: invert())
+		for img in section.xpath("//img[starts-with(@src, '../images/')]"):
+			src = img.get_attr("src").replace("../", "")
+			with open(self.path / "src" / "epub" / src, "rb") as binary_file:
+				image_contents_base64 = base64.b64encode(binary_file.read()).decode()
+
+			if src.endswith(".svg"):
+				img.set_attr("src", f"data:image/svg+xml;base64, {image_contents_base64}")
+
+			if src.endswith(".jpg"):
+				img.set_attr("src", f"data:image/jpg;base64, {image_contents_base64}")
+
+			if src.endswith(".png"):
+				img.set_attr("src", f"data:image/png;base64, {image_contents_base64}")
+
+		for child in section.lxml_element:
+			tag_name = child.tag
+			if tag_name in ("section", "article"):
+				self._recompose_xhtml(se.easy_xml.EasyXmlElement(child), output_dom)
+			else:
+				existing_section.append(se.easy_xml.EasyXmlElement(child))
 
 	def recompose(self, output_xhtml5: bool) -> str:
 		"""
@@ -387,22 +374,21 @@ class SeEpub:
 		A string of HTML5 representing the entire recomposed ebook.
 		"""
 
-		# Get the ordered list of spine items
-		with open(self.metadata_file_path, "r", encoding="utf-8") as file:
-			metadata_soup = BeautifulSoup(file.read(), "lxml")
-
 		# Get some header data: title, core and local css
-		title = html.escape(metadata_soup.find("dc:title").contents[0])
+		title = self.metadata_dom.xpath("//dc:title/text()")[0]
+		language = self.metadata_dom.xpath("//dc:language/text()")[0]
 		css = ""
-		with open(self.path / "src" / "epub" / "css" / "core.css", "r", encoding="utf-8") as file:
-			css = file.read()
+		for filename in os.scandir(self.path / "src" / "epub" / "css"):
+			filepath = Path(filename)
+			if filepath.suffix == ".css":
+				with open(filepath, "r", encoding="utf-8") as file:
+					css = css + f"\n\n\n/* {filepath.name} */" + file.read()
 
-		with open(self.path / "src" / "epub" / "css" / "local.css", "r", encoding="utf-8") as file:
-			css = css + "\n\n\n/* local.css */" + file.read()
+		css = css.strip()
 
 		namespaces = set(regex.findall(r"@namespace.+?;", css))
 
-		css = regex.sub(r"@(charset|namespace).+?;", "", css).strip()
+		css = regex.sub(r"\s*@(charset|namespace).+?;\s*", "\n", css).strip()
 
 		if namespaces:
 			css = "\n" + css
@@ -412,39 +398,50 @@ class SeEpub:
 
 		css = "\t\t\t".join(css.splitlines(True))
 
-		output_xhtml = "<?xml version=\"1.0\" encoding=\"utf-8\"?><html xmlns=\"http://www.w3.org/1999/xhtml\" xmlns:epub=\"http://www.idpf.org/2007/ops\" epub:prefix=\"z3998: http://www.daisy.org/z3998/2012/vocab/structure/, se: https://standardebooks.org/vocab/1.0\"><head><meta charset=\"utf-8\"/><title>" + title + "</title><style/></head><body></body></html>"
-		output_soup = BeautifulSoup(output_xhtml, "lxml")
+		# Remove min-height from CSS since it doesn't really apply to the single page format.
+		# It occurs at least in se.css
+		css = regex.sub(r"\s*min-height: [^;]+?;", "", css)
+
+		# Remove -epub-* CSS as it's invalid in a browser context
+		css = regex.sub(r"\s*\-epub\-[^;]+?;", "", css)
+
+		output_xhtml = f"<?xml version=\"1.0\" encoding=\"utf-8\"?><html xmlns=\"http://www.w3.org/1999/xhtml\" xmlns:epub=\"http://www.idpf.org/2007/ops\" epub:prefix=\"z3998: http://www.daisy.org/z3998/2012/vocab/structure/, se: https://standardebooks.org/vocab/1.0\" xml:lang=\"{language}\"><head><meta charset=\"utf-8\"/><title>{title}</title><style/></head><body></body></html>"
+		output_dom = se.formatting.EasyXhtmlTree(output_xhtml)
 
 		# Iterate over spine items in order and recompose them into our output
-		for element in metadata_soup.select("spine itemref"):
-			filename = metadata_soup.select(f"item[id=\"{element['idref']}\"]")[0]["href"]
+		for ref in self.metadata_dom.xpath("/package/spine/itemref/@idref"):
+			filename = self.metadata_dom.xpath(f"/package/manifest/item[@id='{ref}']/@href")[0]
 
 			with open(self.path / "src" / "epub" / filename, "r", encoding="utf-8") as file:
-				xhtml_soup = BeautifulSoup(file.read(), "lxml")
+				dom = se.formatting.EasyXhtmlTree(file.read())
 
-				for child in xhtml_soup.select("body > *"):
-					self._recompose_xhtml(child, output_soup)
+				for node in dom.xpath("/html/body/*"):
+					self._recompose_xhtml(node, output_dom)
 
 		# Add the ToC after the titlepage
 		with open(self.path / "src" / "epub" / "toc.xhtml", "r", encoding="utf-8") as file:
-			toc_soup = BeautifulSoup(file.read(), "lxml")
-			output_soup.select("#titlepage")[0].insert_after(toc_soup.find("nav"))
+			toc_dom = se.formatting.EasyXhtmlTree(file.read())
+			titlepage_node = output_dom.xpath("//*[contains(concat(' ', @epub:type, ' '), ' titlepage ')]")[0]
+
+			for node in toc_dom.xpath("//nav[1]"):
+				titlepage_node.lxml_element.addnext(node.lxml_element)
+
+		# Replace all <a href> links with internal links
+		for link in output_dom.xpath("//a[contains(@href, '#')]"):
+			link.set_attr("href", regex.sub(r".+(#.+)$", r"\1", link.get_attr("href")))
+
+		# Replace all <a href> links to entire files
+		for link in output_dom.xpath("//a[not(contains(@href, '#'))]"):
+			href = link.get_attr("href")
+			href = regex.sub(r".+/([^/]+)$", r"#\1", href)
+			href = regex.sub(r"\.xhtml$", "", href)
+			link.set_attr("href", href)
 
 		# Get the output XHTML as a string
-		output_xhtml = str(output_soup)
+		output_xhtml = output_dom.to_string()
 		output_xhtml = regex.sub(r"\"(\.\./)?text/(.+?)\.xhtml\"", "\"#\\2\"", output_xhtml)
 		output_xhtml = regex.sub(r"\"(\.\./)?text/.+?\.xhtml#(.+?)\"", "\"#\\2\"", output_xhtml)
 
-		# Replace SVG images hrefs with inline SVG
-		for match in regex.findall(r"src=\"../images/(.+?)\.svg\"", output_xhtml):
-			with open(self.path / "src" / "epub" / "images" / (match + ".svg"), "r", encoding="utf-8") as file:
-				svg = file.read()
-
-				# Remove XML declaration
-				svg = regex.sub(r"<\?xml.+?\?>", "", svg)
-
-				output_xhtml = regex.sub(fr"<img.+?src=\"\.\./images/{match}\.svg\".*?/>", svg, output_xhtml)
-
 		# All done, clean the output
 		output_xhtml = se.formatting.format_xhtml(output_xhtml)
 
@@ -454,6 +451,11 @@ class SeEpub:
 		if output_xhtml5:
 			output_xhtml = output_xhtml.replace("\t\t<meta charset=\"utf-8\"/>\n", "")
 			output_xhtml = output_xhtml.replace("\t\t<style/>\n", "")
+
+			output_xhtml = regex.sub(r'xml:lang="([^"]+?)"', r'xml:lang="\1" lang="\1"', output_xhtml)
+
+			# Re-add a doctype
+			output_xhtml = output_xhtml.replace("<?xml version=\"1.0\" encoding=\"utf-8\"?>", "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<!DOCTYPE html>")
 		else:
 			# Remove xml declaration and re-add the doctype
 			output_xhtml = regex.sub(r"<\?xml.+?\?>", "<!doctype html>", output_xhtml)
@@ -462,9 +464,9 @@ class SeEpub:
 			# Make some replacements for HTML5 compatibility
 			output_xhtml = output_xhtml.replace("epub:type", "data-epub-type")
 			output_xhtml = output_xhtml.replace("epub|type", "data-epub-type")
-			output_xhtml = output_xhtml.replace("xml:lang", "lang")
-			output_xhtml = output_xhtml.replace("<html", f"<html lang=\"{metadata_soup.find('dc:language').string}\"")
+			output_xhtml = output_xhtml.replace("<html", f"<html lang=\"{self.metadata_dom.xpath('/package/metadata/dc:language/text()')[0]}\"")
 			output_xhtml = regex.sub(" xmlns.+?=\".+?\"", "", output_xhtml)
+			output_xhtml = output_xhtml.replace("xml:lang", "lang")
 
 		return output_xhtml
 
@@ -553,9 +555,10 @@ class SeEpub:
 			endnotes_filename = source_directory / "epub/text/endnotes.xhtml"
 			with open(endnotes_filename, "r+", encoding="utf-8") as file:
 				xhtml = file.read()
-				soup = BeautifulSoup(xhtml, "lxml")
 
-				endnote_count = len(soup.select("li[id^=note-]"))
+				dom = se.easy_xml.EasyXhtmlTree(xhtml)
+
+				endnote_count = len(dom.xpath("//li[starts-with(@id, 'note-')]"))
 
 				if increment:
 					note_range = range(endnote_count, target_endnote_number - 1, -1)
@@ -948,53 +951,49 @@ class SeEpub:
 			file_path = self.path / "src/epub/text" / file_name
 			try:
 				with open(file_path) as file:
-					soup = BeautifulSoup(file.read(), "lxml")
+					dom = se.easy_xml.EasyXhtmlTree(file.read())
 			except Exception as ex:
 				raise se.InvalidFileException(f"Couldn’t open file: [path][link=file://{file_path}]{file_path}[/][/].") from ex
 
-			links = soup.find_all("a")
 			needs_rewrite = False
-			for link in links:
-				epub_type = link.get("epub:type") or ""
-				if epub_type == "noteref":
-					old_anchor = ""
-					href = link.get("href") or ""
-					if href:
-						# Extract just the anchor from a URL (ie, what follows a hash symbol)
-						old_anchor = ""
+			for link in dom.xpath("/html/body//a[contains(@epub:type, 'noteref')]"):
+				old_anchor = ""
+				href = link.get_attr("href") or ""
+				if href:
+					# Extract just the anchor from a URL (ie, what follows a hash symbol)
+					hash_position = href.find("#") + 1  # we want the characters AFTER the hash
+					if hash_position > 0:
+						old_anchor = href[hash_position:]
 
-						hash_position = href.find("#") + 1  # we want the characters AFTER the hash
-						if hash_position > 0:
-							old_anchor = href[hash_position:]
+				new_anchor = f"note-{current_note_number:d}"
+				if new_anchor != old_anchor:
+					change_list.append(f"Changed {old_anchor} to {new_anchor} in {file_name}")
+					notes_changed += 1
+					# Update the link in the dom
+					link.set_attr("href", f"endnotes.xhtml#{new_anchor}")
+					link.set_attr("id", f"noteref-{current_note_number:d}")
+					link.lxml_element.text = str(current_note_number)
+					needs_rewrite = True
 
-					new_anchor = f"note-{current_note_number:d}"
-					if new_anchor != old_anchor:
-						change_list.append(f"Changed {old_anchor} to {new_anchor} in {file_name}")
-						notes_changed += 1
-						# Update the link in the soup object
-						link["href"] = 'endnotes.xhtml#' + new_anchor
-						link["id"] = f'noteref-{current_note_number:d}'
-						link.string = str(current_note_number)
-						needs_rewrite = True
-					# Now try to find this in endnotes
-					match_old = lambda x, old=old_anchor: x.anchor == old
-					matches = list(filter(match_old, self.endnotes))
-					if not matches:
-						raise se.InvalidInputException(f"Couldn’t find endnote with anchor [attr]{old_anchor}[/].")
-					if len(matches) > 1:
-						raise se.InvalidInputException(f"Duplicate anchors in endnotes file for anchor [attr]{old_anchor}[/].")
-					# Found a single match, which is what we want
-					endnote = matches[0]
-					endnote.number = current_note_number
-					endnote.matched = True
-					# We don't change the anchor or the back ref just yet
-					endnote.source_file = file_name
-					current_note_number += 1
+				# Now try to find this in endnotes
+				match_old = lambda x, old=old_anchor: x.anchor == old
+				matches = list(filter(match_old, self.endnotes))
+				if not matches:
+					raise se.InvalidInputException(f"Couldn’t find endnote with anchor [attr]{old_anchor}[/].")
+				if len(matches) > 1:
+					raise se.InvalidInputException(f"Duplicate anchors in endnotes file for anchor [attr]{old_anchor}[/].")
+				# Found a single match, which is what we want
+				endnote = matches[0]
+				endnote.number = current_note_number
+				endnote.matched = True
+				# We don't change the anchor or the back ref just yet
+				endnote.source_file = file_name
+				current_note_number += 1
 
 			# If we need to write back the body text file
 			if needs_rewrite:
 				new_file = open(file_path, "w")
-				new_file.write(se.formatting.format_xhtml(str(soup)))
+				new_file.write(se.formatting.format_xhtml(dom.to_string()))
 				new_file.close()
 
 		if processed == 0:
@@ -1002,29 +1001,22 @@ class SeEpub:
 
 		if notes_changed > 0:
 			# Now we need to recreate the endnotes file
-			ol_tag = self._endnotes_soup.ol
-			ol_tag.clear()
+			for ol_tag in self._endnotes_dom.xpath("/html/body/section[contains(@epub:type, 'endnotes')]/ol[1]"):
+				for node in ol_tag.xpath("./li[contains(@epub:type, 'endnote')]"):
+					node.remove()
 
-			self.endnotes.sort(key=lambda endnote: endnote.number)
+				self.endnotes.sort(key=lambda endnote: endnote.number)
 
-			for endnote in self.endnotes:
-				if endnote.matched:
-					li_tag = self._endnotes_soup.new_tag("li")
-					li_tag["id"] = "note-" + str(endnote.number)
-					li_tag["epub:type"] = "endnote"
-					for content in endnote.contents:
-						if isinstance(content, Tag):
-							links = content.find_all("a")
-							for link in links:
-								epub_type = link.get("epub:type") or ""
-								if epub_type == "backlink":
-									href = link.get("href") or ""
-									if href:
-										link["href"] = endnote.source_file + "#noteref-" + str(endnote.number)
-						li_tag.append(content)
-					ol_tag.append(li_tag)
+				for endnote in self.endnotes:
+					if endnote.matched:
+						endnote.node.set_attr("id", f"note-{endnote.number}")
+
+						for node in endnote.node.xpath(".//a[contains(@epub:type, 'backlink')]"):
+							node.set_attr("href", f"{endnote.source_file}#noteref-{endnote.number}")
+
+						ol_tag.append(endnote.node)
 
 			with open(self.path / "src" / "epub" / "text" / "endnotes.xhtml", "w") as file:
-				file.write(se.formatting.format_xhtml(str(self._endnotes_soup)))
+				file.write(se.formatting.format_xhtml(self._endnotes_dom.to_string()))
 
 		return (current_note_number - 1, notes_changed)
