@@ -20,7 +20,7 @@ import importlib_resources
 
 from cairosvg import svg2png
 from natsort import natsorted
-from PIL import Image
+from PIL import Image, ImageOps
 import lxml.cssselect
 import lxml.etree as etree
 import regex
@@ -488,6 +488,8 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						# See The Man Who Was Thursday by G K Chesterton
 						processed_xhtml = regex.sub(r"(<h[1-6] [^>]*) role=\".*?\">", "\\1>", processed_xhtml)
 
+
+
 						# Google Play Books chokes on https XML namespace identifiers (as of at least 2017-07)
 						processed_xhtml = processed_xhtml.replace("https://standardebooks.org/vocab/1.0", "http://standardebooks.org/vocab/1.0")
 
@@ -525,6 +527,20 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 						processed_xhtml = processed_xhtml.replace(" >", ">")
 						processed_xhtml = regex.sub(r"""\s*epub:type=""\s*""", "", processed_xhtml)
 
+						# Move quotation marks over periods and commas
+						processed_xhtml = regex.sub(r"([\\.,])([’”])", r"""\1<span class="quote-align">\2</span>""", processed_xhtml)
+
+						# The above replacement may replace text within <img alt> attributes. Remove those now until no replacements remain, since we may have
+						# many matches in the same line
+						replacements = 1
+						while replacements > 0:
+							processed_xhtml, replacements = regex.subn(r"alt=\"([^<>\"]+?)<span class=\"quote-align\">([^<>\"]+?)</span>", r"""alt="\1\2""", processed_xhtml)
+
+						# Do the same for <title> elements
+						replacements = 1
+						while replacements > 0:
+							processed_xhtml, replacements = regex.subn(r"<title>([^<>]+?)<span class=\"quote-align\">([^<>]+?)</span>", r"""<title>\1\2""", processed_xhtml)
+
 						if processed_xhtml != xhtml:
 							file.seek(0)
 							file.write(processed_xhtml)
@@ -695,10 +711,27 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 										if regex.findall("</?(?:m:)?m", processed_line):
 											# Failure! Abandon all hope, and use Firefox to convert the MathML to PNG.
 											se.images.render_mathml_to_png(driver, regex.sub(r"<(/?)m:", "<\\1", line), work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}.png", work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}-2x.png")
+											# iBooks srcset bug: once srcset works in iBooks, this block can go away
+											# calculate the "normal" height/width from the 2x image
+											ifile = work_epub_root_directory / "epub" / "images" / f"mathml-{mathml_count}-2x.png"
+											image = Image.open(ifile)
+											img_width = image.size[0]
+											img_height = image.size[1]
+											# if either dimension is odd, add a pixel
+											right = img_width % 2
+											bottom = img_height % 2
+											# if either dimension was odd, expand the canvas
+											if (right != 0 or bottom != 0):
+												border = (0, 0, right, bottom)
+												image = ImageOps.expand(image, border)
+												image.save(ifile)
+											# get the "display" dimensions
+											img_width = img_width // 2
+											img_height = img_height // 2
 
 											# iBooks srcset bug: once srcset works in iBooks, we can use this line instead of the one below it
 											# processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}.png\" srcset=\"../images/mathml-{mathml_count}-2x.png 2x, ../images/mathml-{mathml_count}.png 1x\"/>")
-											processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}.png\"/>")
+											processed_xhtml = processed_xhtml.replace(line, f"<img class=\"mathml epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\" src=\"../images/mathml-{mathml_count}-2x.png\" height=\"{img_height}\" width=\"{img_width}\"/>")
 											mathml_count = mathml_count + 1
 										else:
 											# Success! Replace the MathML with our new string.
@@ -752,7 +785,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 		# Convert the <nav> landmarks element to the <guide> element in content.opf
 		guide_xhtml = "<guide>"
 		for element in toc_tree.xpath("//nav[@epub:type=\"landmarks\"]/ol/li/a"):
-			element_xhtml = element.tostring()
+			element_xhtml = element.to_string()
 			element_xhtml = regex.sub(r"epub:type=\"([^\"]*)(\s*frontmatter\s*|\s*backmatter\s*)([^\"]*)\"", "type=\"\\1\\3\"", element_xhtml)
 			element_xhtml = regex.sub(r"epub:type=\"[^\"]*(acknowledgements|bibliography|colophon|copyright-page|cover|dedication|epigraph|foreword|glossary|index|loi|lot|notes|preface|bodymatter|titlepage|toc)[^\"]*\"", "type=\"\\1\"", element_xhtml)
 			element_xhtml = element_xhtml.replace("type=\"copyright-page", "type=\"copyright page")
@@ -850,7 +883,7 @@ def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, outpu
 					node.unwrap()
 
 				file.seek(0)
-				file.write(dom.tostring())
+				file.write(dom.to_string())
 				file.truncate()
 
 			# Rebuild the NCX
