@@ -8,10 +8,10 @@ from pathlib import Path
 from typing import Dict, Optional
 import regex
 import smartypants
-from bs4 import BeautifulSoup
 from hyphen import Hyphenator
 from hyphen.dictools import list_installed
 import se
+from se.formatting import EasyXhtmlTree
 
 
 def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
@@ -32,7 +32,7 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 
 		# First, convert entities.  Sometimes Gutenberg has entities instead of straight quotes.
 		xhtml = html.unescape(xhtml) # This converts html entites to unicode
-		xhtml = regex.sub(r"&([^#\p{Lowercase_Letter}])", r"&amp;\1", xhtml) # Oops!  html.unescape also unescapes plain ampersands...
+		xhtml = regex.sub(r"&(?![#\p{Lowercase_Letter}]+;)", "&amp;", xhtml) # Oops!  html.unescape also unescapes plain ampersands...
 
 		# Replace rsquo character with an escape sequence. We can't use HTML comments
 		# because rsquo may appear inside alt attributes, and that would break smartypants.
@@ -43,11 +43,14 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 
 		# Convert entities again
 		xhtml = html.unescape(xhtml) # This converts html entites to unicode
-		xhtml = regex.sub(r"&([^#\p{Lowercase_Letter}])", r"&amp;\1", xhtml) # Oops!  html.unescape also unescapes plain ampersands...
+		xhtml = regex.sub(r"&(?![#\p{Lowercase_Letter}]+;)", "&amp;", xhtml) # Oops!  html.unescape also unescapes plain ampersands...
 
 	# Replace no-break hyphen with regular hyphen
 	xhtml = xhtml.replace(se.NO_BREAK_HYPHEN, "-")
 
+	# Replace horizontal bar with em dash
+	xhtml = xhtml.replace("―", "—")
+
 	# Replace sequential em dashes with the two or three em dash character
 	xhtml = xhtml.replace("———", "⸻")
 	xhtml = xhtml.replace("——", "⸺")
@@ -64,7 +67,7 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# Remove spaces between en and em dashes
 	# Note that we match at least one character before the dashes, so that we don't catch start-of-line em dashes like in poetry.
 	# We do a negative lookbehind for <br/ to prevent newlines/indents after <br/>s from being included
-	xhtml = regex.sub(r"(?<!<br/)([^\.\s])\s*([–—])\s*", r"\1\2", xhtml, flags=regex.DOTALL)
+	xhtml = regex.sub(r"(?<!<br/)([^\.…\s])\s*([–—])\s*", r"\1\2", xhtml, flags=regex.DOTALL)
 
 	# First, remove stray word joiners
 	xhtml = xhtml.replace(se.WORD_JOINER, "")
@@ -114,6 +117,9 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# in case Mr. is run up against an em dash.
 	xhtml = regex.sub(fr"\b(Mr|Mr?s|Drs?|Profs?|Lieut|Fr|Lt|Capt|Pvt|Esq|Mt|St|MM|Mmes?|Mlles?)\.?(</abbr>)?([^\P{{Punctuation}}\.]|{se.WORD_JOINER})", r"\1.\2\3", xhtml)
 
+	# We added an nbsp after St. above. But, sometimes a name can be abbreviated, like `Bob St. M.`. In this case we don't want an nbsp because <abbr> is already `white-space: nowrap;`.
+	xhtml = regex.sub(fr"""<abbr class="name">St\.{se.NO_BREAK_SPACE}([A-Z])""", r"""<abbr class="name">St\. \1""", xhtml)
+
 	xhtml = regex.sub(r"\bNo\.\s+([0-9]+)", fr"No.{se.NO_BREAK_SPACE}\1", xhtml)
 	xhtml = regex.sub(r"<abbr>No\.</abbr>\s+", fr"<abbr>No.</abbr>{se.NO_BREAK_SPACE}", xhtml)
 
@@ -248,6 +254,13 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# Remove spaces between ellipses and noterefs
 	xhtml = regex.sub(r""" … (<a[^>]+?epub:type="noteref">)""", r" …\1", xhtml)
 
+	# Add an &nbsp; before &amp;
+	xhtml = regex.sub(r" &amp;", f"{se.NO_BREAK_SPACE}&amp;", xhtml)
+
+	# Remove word joiners and nbsp from img alt attributes
+	for match in regex.findall(fr"alt=\"[^\"]*?[{se.NO_BREAK_SPACE}{se.WORD_JOINER}][^\"]*?\"", xhtml):
+		xhtml = xhtml.replace(match, match.replace(se.NO_BREAK_SPACE, " ").replace(se.WORD_JOINER, ""))
+
 	return xhtml
 
 def hyphenate_file(filename: Path, language: Optional[str], ignore_h_tags: bool = False) -> None:
@@ -287,25 +300,22 @@ def hyphenate(xhtml: str, language: Optional[str], ignore_h_tags: bool = False)
 	"""
 
 	hyphenators: Dict[str, Hyphenator] = {}
-	soup = BeautifulSoup(xhtml, "lxml")
+	dom = EasyXhtmlTree(xhtml)
 
 	if language is None:
 		try:
-			language = str(soup.html["xml:lang"])
-		except Exception:
-			try:
-				language = str(soup.html["lang"])
-			except Exception:
-				raise se.InvalidLanguageException("No [attr]xml:lang[/] or [attr]lang[/] attribute on [xhtml]<html>[/] element; couldn’t guess file language.")
+			language = dom.xpath("/html/@xml:lang | /html/@lang")[0]
+		except Exception as ex:
+			raise se.InvalidLanguageException("No [attr]xml:lang[/] or [attr]lang[/] attribute on [xhtml]<html>[/] element; couldn’t guess file language.") from ex
 
 	try:
 		language = language.replace("-", "_")
 		if language not in hyphenators:
 			hyphenators[language] = Hyphenator(language)
-	except Exception:
-		raise se.MissingDependencyException(f"Hyphenator for language [text]{language}[/] not available.\nInstalled hyphenators: {list_installed()}.")
+	except Exception as ex:
+		raise se.MissingDependencyException(f"Hyphenator for language [text]{language}[/] not available.\nInstalled hyphenators: {list_installed()}.") from ex
 
-	text = str(soup.body)
+	text = dom.xpath("/html/body")[0].inner_xml()
 	result = text
 	word = ""
 	in_tag = False
@@ -356,21 +366,24 @@ def hyphenate(xhtml: str, language: Optional[str], ignore_h_tags: bool = False)
 				new_word = word
 
 				# 100 is the hard coded max word length in the hyphenator module
-				# Check here to avoid an error
-				if len(word) < 100:
+				# But, we can't use len(word) because Unicode chars may be longer than len() expects.
+				# So, we simply catch the exception and continue if that ends up happening
+				try:
 					syllables = hyphenators[language].syllables(word)
 
 					if syllables:
 						new_word = "\u00AD".join(syllables)
 
+				except ValueError:
+					pass
+
 				result = result[:pos - len(word) - 1] + new_word + char + result[pos:]
 				pos = pos + len(new_word) - len(word)
 			word = ""
 
 		pos = pos + 1
 
-	xhtml = regex.sub(r"<body.+<\/body>", "", xhtml, flags=regex.DOTALL)
-	xhtml = xhtml.replace("</head>", f"</head>\n\t{result}")
+	xhtml = regex.sub(r"(<body[^>]+?>).+?<\/body>", fr"\1{result}</body>", xhtml, flags=regex.DOTALL)
 
 	return xhtml
 
